/* ************************************************************************************
 *	Das Modul beinhaltet Funktionen zur Berechnung von Voll- und Neumond.
 * Es handelt sich um keine universell einsetzbaren Funktionen. Die Objekte der
 * Ausgabefelder sind in den Funktionen verwendet worden. Sie mssen angepasst
 * oder durch Variablen ersetzt werden.
 * Die Funktionen und Algorithmen haben keinen Anspruch auf
 * hohe Genauigkeit, Vollstndigkeit und Richtigkeit.
 *
 * The module contains functions for calculating full and new moon. 
 * These are not universally applicable functions. The objects of the 
 * output fields have been used in the functions. They must be adjusted 
 * or replaced by variables. 
 * The functions and algorithms have no claim to 
 * high accuracy, completeness and correctness.
 *
 *	Funktionsliste- Function list:
 *	--------------------------------------
 *
 *	- fullMoon()						- Berechnet das JD des Vollmonds.
 *											- Calculates the JD of Full moon
 *  - newMoon()						- Berechnet das JD des Neumonds.
 *											- Calculates the JD of New moon
 *
 *	27.11.2020, Dieter Steffan
 *
 *********************************************************************************** */
 
 
/* *********************************************************************************
 * Die Funktion berechnet das Datum und die lokale Uhrzeit des Neumonds
 * anhand der bergabeparameter als Julianisches Datum UT. Das UT muss
 * danach noch weiter nach LT konvertiert werden.
 * Alle Winkelfunktionen wie Math.sin(Radian); werden hier mit
 * Werten als Radianten versorgt.
 *
 * bergabeparameter:
 * 			dst		=	Sommer/Winterzeit (Daylight Savin Time) als Integer
 * 			tz			=	Zeitzone (Timezone) als Integer
 * 			day		=	Tag der Berechnung als Integer
 * 			month	=	Monat der Berechnung als Integer
 * 			year	=	Jahr der Berechnung als Integer
 *
 *	Returnparameter:
 *						= Datum UT als Julianisches Datum als Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the date and local time of the newmoon based on
 * the handover parameters as the Julian date UT. The UT must:
 * then be further converted to LT.
 * All angular functions such as Math. sin (Radian); are available here with
 * Values supplied as radiants.
 *
 * Handover parameters:
 * 			dst 		= summer/winter time (Daylight Savin Time) as integer
 * 			tz 		= Time zone (Timezone) as integer
 * 			day 		= day of calculation as integer
 * 			month 	= month of calculation as integer
 * 			year 	= year of calculation as integer
 *
 * Returnparameter:
 * 						= Date UT as Julian Date as Number
 *************************************************************************************** */
function newMoon(dst, tz, day, month, year, oJD, oJD0){
	var DST = parseInt(dst);						// bergabeparameter in richtigen  Datentypen wandeln
																	// Convert transfer parameters to correct data types
	var TZ = parseInt(tz);							// Die Zeitzone - The timezone
	var DAY = parseInt(day);						// Der Tag der Berechnung - The day of the calculation
	var MONTH = parseInt(month); 			// Der Monat der Berechnung - The month of the calculation
	var YEAR = parseInt(year);					// Das Jahr der Berechnung- The year of the calculation
	var JD = 0.0;											// Das Julianische Datum des Berechnungsdatums
																	// The Julian date of the calculation date
	var day0 = 0.0;											// Der Tag des Berechnungsdatums
																	// The day of the calculation date
	var month0 = 0.0;										// Der Monat des Berechnungsdatums
																	// The month of the calculation date
	var year0 = 0.0;										// Das Jahr des Berechnungsdatums
																	// The year of the calculation date
	var J0 = 0.0;											// MJD am 0. Januar Berechnungsjahr 00:00:00.000 TZ=0 DST=0
																	// MJD ist 0. Januar year of calculation 00:00:00. 000 TZ=0 DST=0
	var DJ = 0.0;											// MJD am Berechnungsdatum
																	// MJD year of calculation
	var  K = 0.0;												// Ab hier Hilfsvariablen der Berechnung
																	// From here auxiliary variables of the calculation
	// Interne Variablen
	// Internal variables
	var TN = 0.0; var TF = 0.0; var T= 0.0; var T2 = 0.0; var  E = 0.0;
	var c = 0.0; var B = 0.0; var A = 0.0; var A1 = 0.0; var A2 = 0.0;
	var F = 0.0; var DD = 0.0; var NI = 0.0; var NF = 0.0; var B1 = 0.0;
	var NB = 0.0; var FI = 0.0; var FF = 0.0; var FB = 0.0; var newMoon = 0.0;
			
	/*
		Hier wird das Eingabedatum einmal, ber das Julianische Datum, hin-
		und her gerechnet. Dies ist notwendig, da das Eingabedatum der Berechnung
		eine Differenz zum Startdatum der Berechnung aufweisen kann. Dies wird durch die
		Parameter Sommer/Winterzeit (DST) und Zeitzone (TZ) hervorgerufen.
		Die Differenzen sind besonders relevant bei Monats- und Jahresbergngen, da das
		Berechnungsdatum ins Vorjahr oder Vormonat rutschen kann.
								
		Z.B. Eingabedatum 			01.01.2020 DST=0 TZ =-12
		         Berechnungsdatum 	02.02.2020
		-----------------------------------------------------------------------------------------------------------------------------
		Here the input date is once, over the Julian date,
		and reckoned back. This is necessary because the input date of the calculation
		may have a difference to the start date of the calculation. This is due to the
		Parameters summer/winter time (DST) and time zone (TZ). 
		The differences are particularly relevant in the case of monthly and annual transitions, as the
		Calculation date can slide into the previous year or month. 
								
		E. g. input date 					01. 01. 2020 DST=0 TZ=-12
		Calculation date: 				02. 02. 2020
	*/											
	// Julianisches Datum des Berechnungsdatum um 12:00:00.000 Uhr
	// mit Sommer/Winterzeit und Zeitzone berechnen ...
	// Julian date of the calculation date at 12:00:00. 000
	// calculate with summer/winter time and time zone . . .
	JD = calcJD(DAY, MONTH, YEAR, 12,  0, 0, 0, DST,  TZ);
	// Ausgabe des JD, nur zur Kontrolle.
	// Output of the JD, only for control.
	oJD.value = JD;
	// Aktuelles Julianisches Datum mit der Funktion calcJD2Date() und dem
	// Parameter des aktuellen Julianischen Datums	(JD)	in 
	// Datumsformat als Array umwandeln ...
	//
	// Array[0] = Das Datum als Gleitkommazahl. 
	// Dezimalstellen sind die Zeit. 
	// Array[1] = Der Monat des Datums als Ganzzahl. 
	// Array[2] = Das Jahr des Datums als Ganzzahl.
	//
	// Current Julian date iwith the function calcJD2Date () and the
	// Parameters of the current Julian date (JD) in
	// Convert date format as an array . . .
	//
	// Array[0] = The date as floating point number. 
	// Decimal places are the time. 
	// Array[1] = The month of the date as an integer. 
	// Array[2] = The year of the date as an integer. 
	var jdDate = calcJD2Date(JD);
	day0 = Math.floor(jdDate[0]);
	month0 = Math.floor(jdDate[1]);
	year0 = Math.floor(jdDate[2] );							
	// Wenn das Jahr negativ wird + 1. Es muss das Jahr nur immer um
	// 1 Jahr grer werden.
	// If the year becomes negative + 1. It just always has to be around the year
	// grow 1 year.
	if(year0 < 0)year0 += 1;					
	// Ausgabe des Berechnungsdatums, nur zur Kontrolle.
	// Output of the  calculating date, only for control.
	//objStartDate.value = LPad(Math.floor(jdDate[0]), 2, "0") +  "." +  
									//LPad(Math.floor(jdDate[1]), 2, "0") +  "."  + 
									//LPad(Math.floor(jdDate[2] ), 4, "0");
	// Julianisches Datum des Startdatums  DST=0 und TZ=0, also UT 12:00 Uhr	
	// Julian date of start without DST and TZ, it's UT 12:00 a clock high noon						
	oJD0.value = calcJD(day0, month0, year0, 12, 0, 0, 0, 0, 0);
	// ---------------------------------------------------------------------------------------------------------------------------
	// JD = 2415020 = Datum 0.1.1900 12:00 Uhr oder 31.12.1899 12:00 Uhr - TZ=0 DST=0
	// MJD am 0. Januar Berechnungsjahr
	J0 = calcJD(0, 1, year0, 0, 0,0,0,0,0,0)- 2415020;
	// MJD am Berechnungsdatum 00:00:00.000 TZ=0 DST=0
	DJ = calcJD(day0, month0, year0, 0, 0,0,0,0,0,0)- 2415020;					
	// ************************************* Hilfsvariablen der Berechnung ************************
	K = K = lInt(((year0 - 1900.0 + ((DJ - J0) / 365.0)) * 12.3685) + 0.5);
	TN = K / 1236.85;
	TF = (K + 0.5) / 1236.85;
	T = TN;
	// Zur Unterfunktion
	// To sub function
	sechs855();
	// Hier geht es weiter
	NI = A;
	NF = B;
	NB = F;
	T = TF;
	K = K + 0.5;
	// Zur Unterfunktion
	// To the subfunction
	sechs855();
	FI = A;
	FF = B;
	FB = F;
	// Unterfunktion 6855
	// Sub function 6855
	function sechs855(){
		T2 = T * T;
		E = 29.53 * K;
		c = 166.56 + (132.87 - 0.009173 * T) * T;
		c = deg2Radians(c);
		B = 0.00058868 * K + (0.0001178 - 0.000000155 * T) * T2;
		B = B + 0.00033 * Math.sin(c ) + 0.75933;
		A = K / 12.36886;
		A1 = 359.2242 + 360.0 * (A - Math.floor(A)) - (0.0000333 + 0.00000347 * T) * T2;
		A2 = 306.0253 + 360 * ((K / 0.9330851)- Math.floor(K / 0.9330851) );
		A2 = A2 + (0.0107306 + 0.00001236 * T) * T2;
		A = K / 0.9214926;
		F = 21.2964 + 360.0 * decPart(A ) - (0.0016528 + 0.00000239 * T) * T2;
		A1 = unwindDeg(A1);
		A2 = unwindDeg(A2);
		F = unwindDeg(F);
		A1 = deg2Radians(A1);
		A2 = deg2Radians(A2);
		F = deg2Radians(F);
		DD = (0.1734 - 0.000393 * T) * Math.sin(A1 ) + 0.0021 * Math.sin(2 * A1);
		DD = DD - 0.4068 * Math.sin(A2 ) + 0.0161 * Math.sin(2 * A2) - 0.0004 * Math.sin(3 * A2 );
		DD = DD + 0.0104 * Math.sin((2 * F)) - 0.0051 * Math.sin((A1 + A2));
		DD = DD - 0.0074 * Math.sin((A1 - A2)) + 0.0004 * Math.sin((2 * (F + A1)));
		DD = DD - 0.0004 * Math.sin((2 * (F - A1)) ) - 0.0006 * Math.sin((2 * (F + A2))) + 0.001 * Math.sin((2 * (F - A2)));
		DD = DD + 0.0005 * Math.sin((A1 + (2 * A2)));
		E1 = parseInt(E);
		B = B + DD + (E - E1);
		B1 = parseInt(B);
		A = E1 + B1;
		B = B - B1;
	}
	newMoon = NI + 2415020 + NF;
	// Und Returnwert an Caller.
	// And return value to caller.
	return(newMoon);
	
}// End of newMoon()
 
 
 
  /* *********************************************************************************
	Die Funktion berechnet das Julianische Datum des Vollmondes. Der Wert 
	ist aber nicht das Datum und Uhrzeit des Vollmondes.
	Es ist die Universal Time und muss weiter berechnet werden.
					
	bergabewert:					
		dst			= Daylight Saving Time als Integer
		tz				= Timezone als Integer
		day			= Berechnungstag als Integer
		month		= Berechnungsmonat als Integer
		year			= Berechnungsjahr als Integer
				
	Returnwert:
		fullmoon	= Julianisches Datum des UT als Number
	---------------------------------------------------------------------------------------------------
	The function calculates the Julian date of the full moon. But the value is not the 
	date and time of the full moon. It is the Universal Time and must be calculated 
	further. 
	
	Transfer value: 
	
		dst = Daylight Saving Time as integer 
		tz = Timezone as integer 
		day = Calculation day as integer 
		month = Calculation month as integer 
		year = Calculation year as integer 
		
	Return value: 
		fullmoon = Julian date of UT as Number			
  *************************************************************************************** */
 function fullMoon(dst, tz, day, month, year, oJD, oJD0){
	var DST = parseInt(dst);				// bergabeparameter in richtigen  Datentypen wandeln
															// Convert transfer parameters to correct data types
	var TZ = parseInt(tz);					// Die Zeitzone - The timezone
	var DAY = parseInt(day);				// Der Tag der Berechnung - The day of the calculation
	var MONTH = parseInt(month); 	// Der Monat der Berechnung - The month of the calculation
	var YEAR = parseInt(year);			// Das Jahr der Berechnung- The year of the calculation
	var JD = 0.0;									// Das Julianische Datum des Berechnungsdatums
															// The Julian date of the calculation date
	var day0 = 0.0;									// Der Tag des Berechnungsdatums
															// The day of the calculation date
	var month0 = 0.0;								// Der Monat des Berechnungsdatums
															// The month of the calculation date
	var year0 = 0.0;								// Das Jahr des Berechnungsdatums
															// The year of the calculation date
	var J0 = 0.0;									// MJD am 0. Januar Berechnungsjahr 00:00:00.000 TZ=0 DST=0
															// MJD ist 0. Januar year of calculation 00:00:00. 000 TZ=0 DST=0
	var DJ = 0.0;									// MJD am Berechnungsdatum
															// MJD year of calculation
	var  K = 0.0;										// Ab hier Hilfsvariablen der Berechnung
															// From here auxiliary variables of the calculation
	// Interne Variablen
	// Internal variables
	var TN = 0.0; var TF = 0.0; var T= 0.0; var T2 = 0.0; var  E = 0.0;
	var c = 0.0; var B = 0.0; var A = 0.0; var A1 = 0.0; var A2 = 0.0;
	var F = 0.0; var DD = 0.0; var NI = 0.0; var NF = 0.0; var B1 = 0.0;
	var NB = 0.0; var FI = 0.0; var FF = 0.0; var FB = 0.0; var fullMoon = 0.0;
							
	/*
		Hier wird das Eingabedatum einmal, ber das Julianische Datum, hin-
		und her gerechnet. Dies ist notwendig, da das Eingabedatum der Berechnung
		eine Differenz zum Startdatum der Berechnung aufweisen kann. Dies wird durch die
		Parameter Sommer/Winterzeit (DST) und Zeitzone (TZ) hervorgerufen.
		Die Differenzen sind besonders relevant bei Monats- und Jahresbergngen, da das
		Berechnungsdatum ins Vorjahr oder Vormonat rutschen kann.
								
		Z.B. Eingabedatum 			01.01.2020 DST=0 TZ =-12
		Berechnungsdatum 			02.02.2020
		-----------------------------------------------------------------------------------------------------------------------------
		Here the input date is once, over the Julian date,
		and reckoned back. This is necessary because the input date of the calculation
		may have a difference to the start date of the calculation. This is due to the
		Parameters summer/winter time (DST) and time zone (TZ). 
		The differences are particularly relevant in the case of monthly and annual transitions, as the
		Calculation date can slide into the previous year or month. 
								
		E. g. input date 				01. 01. 2020 DST=0 TZ=-12
		Calculation date: 				02. 02. 2020
	*/											
	// Julianisches Datum des Berechnungsdatum um 12:00:00.000 Uhr
	// mit Sommer/Winterzeit und Zeitzone berechnen ...
	// Julian date of the calculation date at 12:00:00. 000
	// calculate with summer/winter time and time zone . . .
	JD = calcJD(DAY, MONTH, YEAR, 12,  0, 0, 0, DST,  TZ);
	// Ausgabe des JD, nur zur Kontrolle.
	// Output of the JD, only for control.
	oJD.value = JD;
	// console.log("JD des Berechnungsdatums=" + JD);
							
	// Aktuelles Julianisches Datum mit der Funktion calcJD2Date() und dem
	// Parameter des aktuellen Julianischen Datums	(JD)	in 
	// Datumsformat als Array umwandeln ...
	//
	// Array[0] = Das Datum als Gleitkommazahl. 
	// Dezimalstellen sind die Zeit. 
	// Array[1] = Der Monat des Datums als Ganzzahl. 
	// Array[2] = Das Jahr des Datums als Ganzzahl.
	//
	// Current Julian date iwith the function calcJD2Date () and the
	// Parameters of the current Julian date (JD) in
	// Convert date format as an array . . .
	//
	// Array[0] = The date as floating point number. 
	// Decimal places are the time. 
	// Array[1] = The month of the date as an integer. 
	// Array[2] = The year of the date as an integer. 
	var jdDate = calcJD2Date(JD);
	day0 = Math.floor(jdDate[0]);
	month0 = Math.floor(jdDate[1]);
	year0 = Math.floor(jdDate[2] );
	// console.log("Eingabetag D0=" + day0);
	// console.log("Eingabetmonat M0=" + month0);
	// console.log("Eingabejahr Y0=" + year0);
							
	// Wenn das Jahr negativ wird + 1. Es muss das Jahr nur immer um
	// 1 Jahr grer werden.
	// If the year becomes negative + 1. It just always has to be around the year
	// grow 1 year.
	if(year0 < 0)year0 += 1;
							
	// Ausgabe des Berechnungsdatums, nur zur Kontrolle.
	// Output of the  calculating date, only for control.
	//oStartDate.value = LPad(Math.floor(jdDate[0]), 2, "0") +  "." +  
								 //LPad(Math.floor(jdDate[1]), 2, "0") +  "."  + 
								 //LPad(Math.floor(jdDate[2] ), 4, "0");
	// Julianisches Datum des Startdatums ohne DST und TZ, also UT 12:00 Uhr	
	// Julian date of start without DST and TZ, it's UT 12:00 a clock high noon						
	oJD0.value = calcJD(day0, month0, year0, 12, 0, 0, 0, 0, 0);
	// ---------------------------------------------------------------------------------------------------------------------------
	// JD = 2415020 = Datum 0.1.1900 12:00 Uhr oder 31.12.1899 12:00 Uhr - TZ=0 DST=0
	// MJD am 0. Januar Berechnungsjahr
	J0 = calcJD(0, 1, year0, 0, 0,0,0,0,0,0)- 2415020;
	// console.log ("MJD Jahresbeginn Berechnungsdatum J0=" + J0);
							
	// MJD am Berechnungsdatum 00:00:00.000 TZ=0 DST=0
	DJ = calcJD(day0, month0, year0, 0, 0,0,0,0,0,0)- 2415020;
	// console.log ("MJD Berechnungsdatum DJ=" + DJ);
							
	// ************************************* Hilfsvariablen der Berechnung ************************
	K = K = lInt(((year0 - 1900.0 + ((DJ - J0) / 365.0)) * 12.3685) + 0.5);
	// console.log ("LINT K=" + K);
	TN = K / 1236.85;
	// console.log ("TN=" + TN);
	TF = (K + 0.5) / 1236.85;
	// console.log ("TF=" + TF);
	T = TN;
	// console.log ("T=" + T);
	// Zur Unterfunktion
	sechs855();
							
	// Hier geht es nach sechs855()weiter
	NI = A;
	// console.log ("Nach 6855 NI=" + NI);
	NF = B;
	// console.log ("Nach 6855 NF=" + NF);
	NB = F;
	// console.log ("Nach 6855 NN=" + NB);
	T = TF;
	// console.log ("Nach 6855 T=" + T);
	K = K + 0.5;
	// console.log ("Nach 6855 K=" + K);
							
	// Zur Unterfunktion
	// To the subfunction
	sechs855();
	FI = A;
	// console.log ("2 x Nach 6855 FI=" + FI);
	FF = B;
	// console.log ("2 x Nach 6855 FF=" + FF);
	FB = F;
	// console.log ("2 x Nach 6855 FB=" + FB);
							
	// Unterfunktion 6855
	// Sub function 6855
	function sechs855(){
		T2 = T * T;
		// console.log ("6855 T2=" + T2);
		E = 29.53 * K;
		// console.log ("6855 E=" + E);
		c = 166.56 + (132.87 - 0.009173 * T) * T;
		// console.log ("6855 c  in Grad=" + c);
		c = deg2Radians(c);
		// console.log ("6855 c  in Rad=" + c);
		B = 0.00058868 * K + (0.0001178 - 0.000000155 * T) * T2;
		// console.log ("6855 B =" + B);
		B = B + 0.00033 * Math.sin(c ) + 0.75933;
		// console.log ("6855 B =" + B);
		A = K / 12.36886;
		// console.log ("6855 A =" + A);
		A1 = 359.2242 + 360.0 * (A - Math.floor(A)) - (0.0000333 + 0.00000347 * T) * T2;
		// console.log ("6855 A1 =" + A1);
		A2 = 306.0253 + 360 * ((K / 0.9330851)- Math.floor(K / 0.9330851) );
		// console.log ("6855 A2 =" + A2);
		A2 = A2 + (0.0107306 + 0.00001236 * T) * T2;
		// console.log ("6855 A2 =" + A2);
		A = K / 0.9214926;
		// console.log ("6855 A =" + A);
		F = 21.2964 + 360.0 * decPart(A ) - (0.0016528 + 0.00000239 * T) * T2;
		// console.log ("6855  F=" + F );
		A1 = unwindDeg(A1);
		// console.log ("6855 UNWIND A1=" + A1);
		A2 = unwindDeg(A2);
		// console.log ("6855 UNWIND A2=" + A2);
		F = unwindDeg(F);
		// console.log ("6855 UNWIND F=" + F);
		A1 = deg2Radians(A1);
		// console.log ("6855 Rad A1=" + A1);
		A2 = deg2Radians(A2);
		// console.log ("6855 Rad A2=" + A2);
		F = deg2Radians(F);
		// console.log ("6855 Rad F=" + F);
		DD = (0.1734 - 0.000393 * T) * Math.sin(A1 ) +  0.0021 * Math.sin(2 * A1);
		DD = DD - 0.4068 * Math.sin(A2 ) +  0.0161 * Math.sin(2 * A2) -  0.0004 * Math.sin(3 * A2 );
		DD = DD + 0.0104 * Math.sin(2  *  F) -  0.0051 * Math.sin(A1 + A2);
		DD = DD - 0.0074 * Math.sin(A1 - A2) +  0.0004 * Math.sin(2 * F + A1);
		DD = DD - 0.0004 * Math.sin(2 * F - A1 ) - 0.0006  *  Math.sin(2 * F + A2) +  0.001 * Math.sin(2 * F - A2);
		DD = DD + 0.0005 * Math.sin(A1 + 2 * A2);
		// console.log ("6855 DD=" + DD);
		E1 = parseInt(E);
		// console.log ("6855 E1=" + E1);
		B = B + DD + (E - E1);
		// console.log ("6855 B=" + B);
		B1 = parseInt(B);
		// console.log ("6855 B1=" + B1);
		A = E1 + B1;
		// console.log ("6855 A=" + A);
		B = B - B1;
		// console.log ("6855 B=" + B);
	}
	fullMoon = FI + 2415020 + FF;
	// console.log ("fullMoon=" + fullMoon);
	// Und Returnwert an Caller.
	// And return value to caller.
	return(fullMoon);
	
}// End of Fullmoon()