/* **********************************************************************************************************
 *	Das Modul beinhaltet einige Konvertierungs Funktionen zwischen den
 * Koordinatensystemen.
 * The module includes some conversion functions between the
 * Coordinate systems .
 *
 *
 *	Funktionsliste - Function list:
 *
 *		- convertEquatorial2Horizontal()		Konvertiert quatrorial Koordinaten zu
 *																		zu horizontal Koordinaten.
 * 																		Convert equatrorial Coordinates to
 * 																		to horizontal coordinates.
 *		- convertEquatorial2Ecliptic()			Berechnet ekliptische Koordinaten aus
 *																		quatorialen Koordinaten.
 *																		Calculates ecliptic coordinates from
 *																		equatorial coordinates.
 *	02.04.2021, Dieter Steffan
 *
 ****************************************************************************************************************/

 
 
 
/* *********************************************************************************
 *	Die Funktion konvertiert quatorial Koordinaten nach Horizontal 
 *	Koordinaten. Dies wird bentigt, um die Hhe und den Azimuth von
 * Objekten, Sterne, Mond, Sonne ... am Himmel anzugeben.
 * Die Funktion darf nicht mit der Funktion objectheight() in derr
 * Berechnungs des Mondauf/Untergangs moonSetAz()moonRiseAz()
 * verwechselt werden.
 *	
 *	bergabewert:
 *		raOrAZDeg	= RA in dezimalen Grad und als Number
 *		decDeg			= Deklination in dezimalen Grad und als Number
 *		glatDeg			= der Breitengrad des Beobachtungsortes in dezimalen Grad
 *									und als Number
 *		
 *	Returnwert:
 *		Array mit den Ergebnissen und Indizes:
 *		[0]	=	Azimuth in deszimalen Grad und als Number
 *		[1]	= Deklination in dezimalen Grad und als Number
 *		
 *	---------------------------------------------------------------------------------------------------
 * The function converts equatorial coordinates to horizontal
 * Coordinates. This is needed to adjust the height and azimuth of
 * Objects, stars, moon, sun . . . in the sky.
 * The function must not be used with the objectheight () function in the
 * Calculation of the moon rise/ Sunset moonSetAz () moonRiseAz ( )
 * to be confused.
 *
 * Transfer value:
 * 		raOrAZDeg = RA in decimal degrees and as number
 * 		decDeg = 		Declination in decimal degrees and as number
 * 		glatDeg = 		the latitude of the observation site in decimal degrees
 * 								and as a number
 *
 * Return value:
 * 		Array with results and indices:
 * 		[ 0] = Azimuth in decimal degrees and as number
 * 		[1 ] = Declination in decimal degrees and as number
  *************************************************************************************** */
function convertEquatorial2Horizontal(raOrAZDeg, decDeg, glatDeg){
	var RET = new Array();										// Der Rckgabewert
																						// The return value
	var RAORAZDEG = Number(raOrAZDeg);		// Der Stundenwinkel RA/AZ in dezimalen Grad
	var DECDEG = Number(decDeg);					//	Die Deklination/Hhe in dezimalen Grad
	var GLATDEG = Number(glatDeg);					// Geographischer Ort des Beobachters
	var SINA = 0.0;															// Der Sinus des Winkels Alpha
	var COSA = 0.0;														// Der Cosinus des Winkels Alpha
	var A = 0.0;																// Der Winkel von SINA
	var ASTRICH = 0.0;													// Der Winkel aus COSA
	var SINH = 0.0;															// Der Sinus der RA/AZ
	var AZH = 0.0;															// Der wahre Azimuth/Hhe in Grad

	// Den Sinus des Winkels Alpha aus RA/AZ und Dec/Hhe und geo. Breite berechnen ...
	// The sine of the angle Alpha of RA/AZ and Dec/Height and geo. Calculate width . . .
	SINA = Math.sin(DECDEG / RAD) * Math.sin(GLATDEG / RAD)+
				 Math.cos(DECDEG / RAD)* Math.cos(GLATDEG / RAD)* Math.cos(RAORAZDEG / RAD);
	// ... und dann den Winkel Alpha in Grad berechnen, das ist die Deklination ...
	// . . . And then calculate the angle alpha in degrees, thats the declination. . .
	A = Math.asin(SINA)* RAD;
	// ... dann den Cosinus des Winkels Alpha aus RA/AZ und Dec/Hhe und geo. Breite berechnen ...
	// . . . then the cosine of the angle Alpha of RA/AZ and Dec/Height and geo. Calculate width . . .
	COSA = (Math.sin(DECDEG / RAD)- Math.sin(GLATDEG / RAD)* Math.sin(A / RAD))/
					(Math.cos(GLATDEG / RAD)* Math.cos(A / RAD));
	// ... und dann den Arcus Cosinus davon in Grad umrechnen ...
	// . . . and then convert the arcus cosine of it into degrees . . .
	ASTRICH = Math.acos(COSA)* RAD;
	// ... dann den Sinus der RA berechnen ...
	// . . . then calculate the sine of the RA . . .
	SINH = Math.sin(RAORAZDEG / RAD);
	// ... und dies ist die Kennziffer fr das Ergebnis des Azimuths zur Deklination ...
	// . . . and this is the index for the result of the azimuth to the declination . . .
	if (SINH < 0.0)
		AZH = ASTRICH;
	else
		AZH = 360.0 - ASTRICH;
	// ... RA/AZ ins Array ...
	// . . . RA/AZ to the array . . .
	RET.push(AZH);
	// Deklination/Hhe ins Array ...
	// Declination /Height to Array . . .
	RET.push(A);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(RET);
	
}// End of convertEquatorial2Horizontal()




/* *********************************************************************************
 *	Die Funktion Konvertiert quatorial Koordinaten nach ekliptischen
 *	Koordinaten. Als Ergebnis wird der Lngengrad Lambda und
 * der Breitengrad Beta berechnet. Bercksichtigt wird die ekliptische
 * Schiefe
 *	
 *	bergabewert:
 *		azDeg				= RA in dezimalen Grad und als Number
 *		decDeg			= Deklination als dezimale Grad und als Number
 *		calcDay			= Berechnungstag
 *		calcMonth		= Berechnungsmonat
 *		calcYear			= Berechnungsjahr
 *		
 *	Returnwert:
 *		Array mit den Ergebnissen und Indizes:
 *		[0]	=	Lngengrad als dezimalen Grad und als Number
 *		[1]	= Breitengrad als dezimalen Grad und als Number
 *	---------------------------------------------------------------------------------------------------
 * Convert equatorial coordinates to ecliptic
	* Coordinates. As a result, the longitude lambda and
	* the latitude beta is calculated. Account is taken of the ecliptic
	* skewed
	*
	* Transfer value:
	* azDeg = RA in decimal degrees and as number
	* decDeg = Declination as a Decimal Degree and as a Number
	* CalcDay = Calculation day
	* CalcMonth = Month of calculation
	* CalcYear = Year of calculation
	*
	* Return value:
	* Array with results and indices:
	* [0] = Longitude as decimal degree and as number
	* [1] = latitude as decimal degrees and as number
  *************************************************************************************** */
function convertEquatorial2Ecliptic(azDeg, decDeg, calcDay, calcMonth, calcYear){
	var ret = new Array();										// Der Rckgabewert
																					// The return value
	var DECDEG = Number(decDeg);				// Der quatoriale Azimuth/RA als Grad und Dezimalzahl
																					// The equatorial azimuth/RA as degrees and decimal value
	var AZDEG = Number(azDeg);						// Die Deklination als Grad und Dezimalzahl
																					// The declination as degree and decimal value
	var CALCDAY = Number(calcDay);				// Der Tag der Kalkulation
																					// Day of calculation
	var CALCMONTH = Number(calcMonth);	// Der Monat der Kalkulation
																					// Month of calculation
	var CALCYEAR = Number(calcYear);			// Das Jahr der Kalkulation
																					// Year of calculation
	var epsilon = 0.0;													// Die ekliptische Schiefe
																					// The ecliptic obliquity
	var sinBeta = 0.0;													// Hilfsvariablen
	var beta = 0.0;														// Auxalary variables
	var y = 0.0;
	var  x = 0.0;
	var lamddaStrich = 0.0;
	// Berechne die eklipitsche Schiefe ...
	// Calculate the ecliptic obliquity ...
	epsilon = calcObliquityEcliptic(calcJD(CALCDAY, CALCMONTH,  CALCYEAR, 0, 0, 0, 0, 0, 0));
	// ... dann den Sinus des Breitengrades ...
	// . . . then the sine of latitude . . .
	sinBeta = Math.sin(AZDEG / RAD)* Math.cos(epsilon / RAD)- 
					 Math.cos(AZDEG / RAD)* Math.sin(epsilon / RAD)* Math.sin(DECDEG / RAD);
	// ... und den Breitengrad in dezimalen Grad ...
	// . . . and the latitude in decimal degrees . . .
	beta = Math.asin( sinBeta)* RAD;
	// ... nun den y-Wert des Lngengrades berechnen ...
	// . . . now calculate the y-value of the longitude . . .
	y = Math.sin(DECDEG / RAD)* Math.cos(epsilon / RAD) +
		  Math.tan(AZDEG / RAD) * Math.sin(epsilon / RAD);
	// ... und den x-Wert ...
	// and then the x value ...
	x = Math.cos(DECDEG / RAD);
	// ... und dann in den dezimalen Langengrad umrechnen ...
	// . . . and then convert to the decimal longitude . . .
	lambdastrich = anotherArcTan(x, y) * RAD;
	// ... Grad Lngengrad ins Array ...
	// ... Degrees longitude to array
	ret.push (lambdastrich);
	// ... Grad Breitengrad ins Array ...
	// ... Degrees altitude to array
	ret.push (beta);
	// ... und zurck zum Aufrufer
	// ... and back to the caller ...
	return(ret);
	
}// End of convertEquatorial2Ecliptic()
